@@ -1,6 +1,5 @@
 import json
 from dataclasses import dataclass
-from typing import List
 
 from mipdb.exceptions import InvalidDataModelError
 
@@ -11,10 +10,10 @@ class CommonDataElement:
     metadata: str
 
     @classmethod
-    def from_cde_data(cls, cde_data):
-        code = cde_data["code"]
-        validate_cde_data(code, cde_data)
-        metadata = json.dumps(cde_data)
+    def from_metadata(cls, metadata):
+        code = metadata["code"]
+        validate_metadata(code, metadata)
+        metadata = json.dumps(metadata)
 
         return cls(
             code,
@@ -29,53 +28,52 @@ def get_system_column_metadata():
         "code": "subjectcode",
         "sql_type": "text",
     }
-    return CommonDataElement.from_cde_data(subjectcode)
+    return CommonDataElement.from_metadata(subjectcode)
 
 
 def make_cdes(schema_data):
     cdes = []
 
     if "variables" in schema_data:
-        for cde_data in schema_data["variables"]:
-            cde_data = reformat_cde_data(cde_data)
-            cdes += [CommonDataElement.from_cde_data(cde_data)]
+        for metadata in schema_data["variables"]:
+            metadata = reformat_metadata(metadata)
+            cdes += [CommonDataElement.from_metadata(metadata)]
     if "groups" in schema_data:
         cdes += [
-            cde_data
+            metadata
             for group_data in schema_data["groups"]
-            for cde_data in make_cdes(group_data)
+            for metadata in make_cdes(group_data)
         ]
     return cdes
 
 
-def reformat_cde_data(cde_data):
+def reformat_metadata(metadata):
     new_key_assign = {
         "isCategorical": "is_categorical",
         "minValue": "min",
         "maxValue": "max",
     }
     for old_key, new_key in new_key_assign.items():
-        if old_key in cde_data:
-            cde_data[new_key] = cde_data.pop(old_key)
+        if old_key in metadata:
+            metadata[new_key] = metadata.pop(old_key)
 
-    if "enumerations" in cde_data:
-        print(type(cde_data["enumerations"]))
-        cde_data["enumerations"] = [
-            {enumeration["code"]: enumeration["label"]}
-            for enumeration in cde_data["enumerations"]
-        ]
-    return cde_data
+    if "enumerations" in metadata:
+        metadata["enumerations"] = {
+            enumeration["code"]: enumeration["label"]
+            for enumeration in metadata["enumerations"]
+        }
+    return metadata
 
 
-def validate_cde_data(code, cde_data):
+def validate_metadata(code, metadata):
     for element in ["is_categorical", "code", "sql_type", "label"]:
-        if element not in cde_data:
+        if element not in metadata:
             raise InvalidDataModelError(
                 f"Element: {element} is missing from the CDE {code}"
             )
-    if cde_data["is_categorical"] and "enumerations" not in cde_data:
+    if metadata["is_categorical"] and "enumerations" not in metadata:
         raise InvalidDataModelError(
             f"The CDE {code} has 'is_categorical' set to True but there are no enumerations."
         )
-    if {"min", "max"} < set(cde_data) and cde_data["min"] >= cde_data["max"]:
+    if {"min", "max"} < set(metadata) and metadata["min"] >= metadata["max"]:
         raise InvalidDataModelError(f"The CDE {code} has min greater than the max.")
